local _, _, REVISION = string.find("$Revision: 17392 $", "(%d+)")

--[[
-- lua 5.1 check
if(not loadstring("return function(...) return ... end")  and true or false) then
	string.gmatch = string.gfind;
    math.fmod = math.mod;    
    table.maxn = table.getn;
end
--]]

local lang = AceLibrary("AceLocale-2.2"):new("SchoolTeacher")

lang:RegisterTranslations("enUS", function() return {
	["Teacher"] = true,

	["Rebuild"] = true,
	["Rebuild teacher data."] = true,
    
	["Advanced Mode"] = true,
	["Toggles advanced teacher options visibility."] = true,    
    
    ["Take Notes"] = true,
    ["Toggle Cartographer note taking."] = true,
} end)

SchoolTeacher = School:NewModule("Teacher")

SchoolTeacher.defaults = {
	advancedMode = false,
    takeNotes = true,
}

SchoolTeacher.options = {
    rebuild = {
               type = 'execute',
               name = lang["Rebuild"],
               desc = lang["Rebuild teacher data."],
               func = function() SchoolTeacher:Rebuild() end,
               order = 110,
    },

	advancedMode = {
		type = "toggle",
		name = lang["Advanced Mode"],
		desc = lang["Toggles advanced teacher options visibility."],
		get = function() return SchoolTeacher:ToggleAdvancedMode() end,
		set = function(v) SchoolTeacher:ToggleAdvancedMode(v) end,
		order = 120,
	},
    
    takeNotes = {
        type = "toggle",
        name = lang["Take Notes"],
        desc = lang["Toggle Cartographer note taking."],
        get = function() return SchoolTeacher.db.profile.takeNotes end,
        set = function(v) SchoolTeacher.db.profile.takeNotes = v end,
        order = 130,
    },
}

SchoolTeacher:SetDebugging(false)
SchoolTeacher:SetDebugLevel(3)

SchoolTeacher.revision = tonumber(REVISION)

function SchoolTeacher:OnInitialize()
end

function SchoolTeacher:OnEnable()
    self:RegisterEvent("TRAINER_SHOW")
    self:RegisterEvent("TRAINER_CLOSED")
    
    self.class = self.core.classid
end

function SchoolTeacher:OnDisable()
end

function SchoolTeacher:GetData()
    local data = {}

    if (not self.db.char.spells) then self:LogTrainerInfo() end
    data.spells = self.db.char.spells
    data.trainers = self.db.char.trainers
    
    return data
end

function SchoolTeacher:ToggleAdvancedMode()
	if type(value) == "nil" then
		return self.db.profile.advancedMode
	end

	self.db.profile.advancedMode = value
end

function SchoolTeacher:TRAINER_SHOW()
    self.DoneAtTrainer = false
end

function SchoolTeacher:TRAINER_CLOSED()
    if (self.DoneAtTrainer == false) then
        self.DoneAtTrainer = true
        self:LogTrainerInfo()
    end
end

function SchoolTeacher:Rebuild()
    self.db.char.spells = nil
    self.db.char.trainers = nil
    self:LogTrainerInfo()
    return self:GetData()
end

function SchoolTeacher:LogTrainerInfo()
    if (IsTradeskillTrainer() == true) then
        self:LogTradeskillTrainerInfo()
    else
        self:LogClassTrainerInfo()
    end
    
    self:TriggerEvent("School_Teacher_Changed")
end

function SchoolTeacher:LogTradeskillTrainerInfo()
    self:LevelDebug(3, "LogTradeskillTrainerInfo")
end

function SchoolTeacher:LogClassTrainerInfo()
    -- self:Debug("LogClassTrainerInfo")
    local spells = {}
    local trainers = {}
    local notes = {}

    if (self.db.char.spells ~= nil) then spells = self.db.char.spells end
    if (self.db.char.trainers ~= nil) then trainers = self.db.char.trainers end
    if (self.db.char.notes ~= nil) then notes = self.db.char.notes end
    
    local npc = {}
    npc.name = self:GetTrainerName()
    npc.zone = self:GetTrainerZone()
    npc.subzone = self:GetTrainerSubZone()
    npc.type = self:GetTrainerType()
    
    local filter = {}
    filter.available = GetTrainerServiceTypeFilter("available")
    filter.unavailable = GetTrainerServiceTypeFilter("unavailable")
    filter.used = GetTrainerServiceTypeFilter("used")
    
    SetTrainerServiceTypeFilter("available", 1)
    SetTrainerServiceTypeFilter("unavailable", 1)
    SetTrainerServiceTypeFilter("used", 1)
    
    ExpandTrainerSkillLine(0)
    
    local numTrainerServices = GetNumTrainerServices()
    
    if (not numTrainerServices or (numTrainerServices < 1)) then
        return
    end
    
    local category = nil
    for i = 1, numTrainerServices do
        local service = {}
        
        service.name, service.subtext, service.type, service.expanded = GetTrainerServiceInfo(i)

        if (not service.name or not service.type) then return end
        if (service.type == "header") then
            category = service.name
        else
            if (category ~= nil) then service.category = category end
            if (not service.subtext) or (string.len(service.subtext) < 1) then service.subtext = "None" end
            service.fullname = service.name
            
            if (service.subtext ~= "None") then
                -- Check for "RANK"
                service.fullname = service.fullname .. " (" .. service.subtext .. ")"
            end
            
            service.cost = {}
            -- Money, Character Point Cost 1, Character Point Cost 2
            service.cost.money, service.cost.cost1, service.cost.cost2 = GetTrainerServiceCost(i)

            service.icon = GetTrainerServiceIcon(i)
            service.level = GetTrainerServiceLevelReq(i)
        
            local req = {}
            req.skill, req.rank, req.hasReq = GetTrainerServiceSkillReq(i)
            if (req.skill) then service.req = req end
            
            local numRequirements = GetTrainerServiceNumAbilityReq(i);
            if (numRequirements > 0) then
                service.ability = {}
                for k = 1, numRequirements do
                    service.ability[k] = {}
                    service.ability[k].skill, service.ability[k].req = GetTrainerServiceAbilityReq(i, k)
                end
            end

            -- Fix wrapped descriptions
            -- service.description = GetTrainerServiceDescription(i)
            
            service.learnspell, service.petlearnspell = IsTrainerServiceLearnSpell(i)
            
            
            if (not spells[service.category]) then spells[service.category] = {} end
            if (not spells[service.category][service.name]) then spells[service.category][service.name] = {} end
            if (not spells[service.category][service.name][service.subtext]) then spells[service.category][service.name][service.subtext] = {} end
            spells[service.category][service.name][service.subtext] = service
            
            if (not trainers[service.name]) then trainers[service.name] = {} end
            if (not trainers[service.name][service.subtext]) then trainers[service.name][service.subtext] = {} end
            if (not trainers[service.name][service.subtext][npc.name]) then trainers[service.name][service.subtext][npc.name] = npc end
        end
    end
    
    if Cartographer_Notes and self.db.profile.takeNotes then
        if (not notes.trainers) then notes.trainers = {} end
        if (not notes.trainers[npc.name]) then notes.trainers[npc.name] = {} end
        if (not notes.trainers[npc.name].noted) then 
            self:NoteIt(npc) 
            notes.trainers[npc.name].noted = true
        end
	end

    self.db.char.spells = spells
    self.db.char.trainers = trainers
    self.db.char.notes = notes
end

function SchoolTeacher:GetTrainerName()
    return UnitName("npc")
end

function SchoolTeacher:GetTrainerZone(trainer)
    return GetZoneText()
end

function SchoolTeacher:GetTrainerSubZone(trainer)
    return GetSubZoneText()
end

function SchoolTeacher:GetTrainerType(trainer)
    if (AceLibrary:HasInstance("Gratuity-2.0")) then
        local tooltip = AceLibrary("Gratuity-2.0")
        
        tooltip:SetUnit("npc")
        -- local _,_, result = tooltip:Find("(%S+) Trainer")    
        local result = tooltip:GetLine(2)
        
        return result
    else 
        if (not trainer) then trainer = self:GetTrainerName() end
    
        local trainerTypes = {}
        trainerTypes["Aldrae"] = "Priest Trainer"
        trainerTypes["Lotheolan"] = "Priest Trainer"
        trainerTypes["Belestra"] = "Priest Trainer"
        trainerTypes["Ponaris"] = "Priest Trainer"

        return trainerTypes[trainer]
    end
end

function SchoolTeacher:LogTalentInfo()
    -- self:LevelDebug(3, "LogTalentInfo")
end

function SchoolTeacher:BuildSkills()
    local skills = {}
end

function SchoolTeacher:NoteIt(trainer)
    -- self:Debug("NoteIt")
    SetMapToCurrentZone()

    local note = {}
    note.zone = GetRealZoneText()
    note.x, note.y = GetPlayerMapPosition("player")
    note.title = trainer.name
    note.titleR = 0
    note.titleG = 1
    note.titleB = 0
    note.info = trainer.type or ""
    note.infoR = 1
    note.infoG = 1
    note.infoB = 1
    note.info2 = ""
    note.info2R = 1
    note.info2G = 1
    note.info2B = 1
    note.creator = "School"
    note.icon = "Interface\\Icons\\INV_Misc_Book_08"

    -- Cartographer_Notes:SetNote(zone, x, y, 
    --                            title, titleR, titleG, titleB, 
    --                            info, infoR, infoG, infoB, 
    --                            info2, info2R, info2G, info2B, 
    --                            creator, icon, manual, oldId)
    
    if (Cartographer_Notes.SetNote ~= nil) then
        Cartographer_Notes:SetNote(
                                   note.zone, note.x, note.y,
                                   note.title, note.titleR, note.titleG, note.titleB,
                                   note.info, note.infoR, note.infoG, note.infoB,
                                   note.info2, note.info2R, note.info2G, note.info2B,
                                   note.creator, note.icon, note.manual, note.oldId
        )
    end
end




