local _, _, REVISION = string.find("$Revision: 17474 $", "(%d+)")
local lang = AceLibrary("AceLocale-2.2"):new("SchoolUI")
local dewdrop = AceLibrary("Dewdrop-2.0")
local abacus = AceLibrary("Abacus-2.0")
local crayon = AceLibrary("Crayon-2.0")
-- local toolbox = AceLibrary("Toolbox-1.0")
-- local deformat = AceLibrary("Deformat-2.0")
-- local tablet = AceLibrary("Tablet-2.0")

lang:RegisterTranslations("enUS", function() return {
	["UI"] = true,

	["Show"] = true,
	["Show school data."] = true,
    
	["Advanced Mode"] = true,
	["Toggles advanced school options visibility."] = true,    
} end)

SchoolUI = School:NewModule("UI")

SchoolUI.defaults = {
	advancedMode = false,
	locked = false,
	fixedTooltip = true,
	resultRows = 10,
	backgroundAlpha = 0.75,
	tooltipPosition = "left",
    detailPosition = "left",
    ignoreCost = true,
    ignoreCombat = false
}

SchoolUI.commands = {
    show = {
            type = 'execute',
            name = lang["Show"],
            desc = lang["Show school data."],
            func = function() SchoolUI:OpenUI() end,
            order = -2,
    }
}

SchoolUI.options = {
                    --[[
                	advancedMode = {
                            		type = "toggle",
                            		name = lang["Advanced Mode"],
                            		desc = lang["Toggles advanced school options visibility."],
                            		get = function() return SchoolUI:ToggleAdvancedMode() end,
                            		set = function(v) SchoolUI:ToggleAdvancedMode(v) end,
                            		order = 120,
                	},
                    --]]
                    lock = {
                            type = "toggle",
                            name = "Lock position",
                            desc = "Toggles locking the School frame in place.",
                            get = function() return SchoolUI.db.profile.locked end,
                            set = function(v) SchoolUI.db.profile.locked = v end,
                            order = 120,
                    },
                    fixedtooltip = {
                            type = "toggle",
                            name = "Fixed Tooltip",
                            desc = "Locks the tooltip in place on the left side of the frame.",
                            get = function() return SchoolUI.db.profile.fixedTooltip end,
                            set = function(v) SchoolUI.db.profile.fixedTooltip = v end,
                            order = 130,
                    },
                    tooltipposition = {
                                       type = "text",
                                       name = "Tooltip Position",
                                       desc = "Sets tooltip placement on left or right of frame.",
                                       get = function() return SchoolUI.db.profile.tooltipPosition end,
                                       set = function(v) SchoolUI.db.profile.tooltipPosition = v end,
                                       validate = {
                                                   ["left"] = "Left",
                                                   ["right"] = "Right",
                                       },
                                       order = 140,
                    },
                    detailposition = {
                                      type = "text",
                                      name = "Detail Position",
                                      desc = "Sets detail placement on left or right of frame.",
                                      get = function() return SchoolUI.db.profile.detailPosition end,
                                      set = function(v) SchoolUI.db.profile.detailPosition = v end,
                                      validate = {
                                                  ["left"] = "Left",
                                                  ["right"] = "Right",
                                      },
                                      order = 150,
                    },
                    
                    rows = {
                            type = "range",
                            name = "Rows",
                            desc = "Sets result row count.\n|cffff0000REQUIRES RELOADUI|r",
                            min = 5,
                            max = 25,
                            step = 1,
                            isPercent = false,
                            get = function() return SchoolUI.db.profile.resultRows end,
                            set = function(v) SchoolUI.db.profile.resultRows = v end,
                            order = 160,
                    },
                    backgroundalpha = {
                                       type = "range",
                                       name = "Backdrop Alpha",
                                       desc = "Sets frame background transparency level.",
                                       min = 0,
                                       max = 1,
                                       step = 0,
                                       isPercent = false,
                                       get = function() return SchoolUI.db.profile.backgroundAlpha end,
                                       set = function(v)
                                          SchoolUI.db.profile.backgroundAlpha = v
                                          SchoolUI:SetBackdropAlpha()
                                       end,
                                       order = 170,
                    },
                    ignorecost = {
                                  type = "toggle",
                                  name = "Ignore cost",
                                  desc = "Ignore learning cost.",
                                  get = function() return SchoolUI.db.profile.ignoreCost end,
                                  set = function(v) 
                                    SchoolUI.db.profile.ignoreCost = v 
                                    SchoolUI.UpdateResults()
                                  end,
                                  order = 180,
                    },
                    ignorecombat = {
                                    type = "toggle",
                                    name = "Ignore combat",
                                    desc = "Ignore combat.",
                                    get = function() return SchoolUI.db.profile.ignoreCombat end,
                                    set = function(v) SchoolUI.db.profile.ignoreCombat = v end,
                                    order = 190,
                    },
                    
}

SchoolUI:SetDebugging(false)
SchoolUI:SetDebugLevel(3)

SchoolUI.revision = tonumber(REVISION)

function SchoolUI:OnInitialize()
	self.resultRows = self.db.profile.resultRows
end

function SchoolUI:OnEnable()
    self:RegisterEvent("CHARACTER_POINTS_CHANGED")
    self:RegisterEvent("PLAYER_LEVEL_UP")
    self:RegisterEvent("PLAYER_MONEY")
    
    self:RegisterEvent("School_Teacher_Changed")
    self:RegisterEvent("School_Student_Changed")
    self:RegisterEvent("School_Changed")
end

function SchoolUI:OnDisable()
end

function SchoolUI:ToggleAdvancedMode()
	if type(value) == "nil" then
		return self.db.profile.advancedMode
	end

	self.db.profile.advancedMode = value
end

function SchoolUI:CHARACTER_POINTS_CHANGED()
    if (not UnitAffectingCombat("player")) then
        self:OpenUI()
    end
    
    if (self.frames) and (self.frames.main:IsShown()) then
        self:UpdateResults()
    end
end

function SchoolUI:PLAYER_LEVEL_UP()
    if (not UnitAffectingCombat("player")) then
        self:OpenUI()
    end
    
    if (self.frames) and (self.frames.main:IsShown()) then
        self:UpdateResults()
    end
end

function SchoolUI:PLAYER_MONEY()
    if (self.frames) and (self.frames.main:IsShown()) then
        self:UpdateResults()
    end
end

function SchoolUI:School_Teacher_Changed()
    -- self:Debug("School_Teacher_Changed")
    if (self.frames) and (self.frames.main:IsShown()) then
        self:UpdateResults()
    end
end

function SchoolUI:School_Student_Changed()
    -- self:Debug("School_Student_Changed")
    if (self.frames) and (self.frames.main:IsShown()) then
        self:UpdateResults()
    end
end

function SchoolUI:School_Changed()
    -- self:Debug("School_Changed")
    if (self.frames) and (self.frames.main:IsShown()) then
        self:UpdateResults()
    end
end

function SchoolUI:OpenUI()
    self:Debug("OpenUI")
    
	self:CreateUI()
    self:UpdateResults()

	if not self.frames.main:IsShown() then
		self.frames.main:Show()
	elseif not dontHide then
		self.frames.main:Hide()
	end
end

function SchoolUI:PlaceTooltip()
	if self.db.profile.tooltipPosition == "left" then
		self.tooltips.show.icon:ClearAllPoints()
		self.tooltips.show.icon:SetPoint("TOPLEFT", self.tooltips.show, "TOPLEFT", -37, -3)

		if self.db.profile.fixedTooltip then
			self.tooltips.show:SetOwner(self.frames.main.items[1], "ANCHOR_BOTTOMLEFT", -2, 43)
		else
			self.tooltips.show:SetOwner(this, "ANCHOR_BOTTOMLEFT")
		end
	else
		self.tooltips.show.icon:ClearAllPoints()
		self.tooltips.show.icon:SetPoint("TOPRIGHT", self.tooltips.show, "TOPRIGHT", 37, -3)

		if self.db.profile.fixedTooltip then
			self.tooltips.show:SetOwner(self.frames.main.items[1], "ANCHOR_BOTTOMRIGHT", 20, 43)
		else
			self.tooltips.show:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
		end
	end
end

function SchoolUI:CreateUI()
    self:Debug("CreateUI")
	if self.frames then return end
    
    self:Debug("self.frames is %s", type(self.frames))

	self.frames = {}
	self.tooltips = {}
    
    local frameWidth = 300
    local buttonWidth = frameWidth - 10
    
  	local f = CreateFrame("Frame", nil, UIParent)
	f:Hide()
	f:SetFrameStrata("BACKGROUND")
	-- f:SetWidth(270)
    f:SetWidth(frameWidth)
	f:SetHeight(29 + 19 * self.resultRows)
	f:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	})
	f:SetClampedToScreen(true)
	f:SetMovable(true)
	f:EnableMouse(true)
	f:RegisterForDrag("LeftButton") 
	f:SetScript("OnDragStart", function()
		if not self.db.profile.locked then
			this:StartMoving()
		end
	end)
	f:SetScript("OnDragStop", function()
		if not self.db.profile.locked then
			this:StopMovingOrSizing()
			self:SavePosition()
		end
	end)
	f:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    
    f.menuButton = CreateFrame("Button", nil, f)
	f.menuButton:SetWidth(240)
	f.menuButton:SetHeight(19)
	f.menuButton:RegisterForDrag("LeftButton")
	f.menuButton:SetScript("OnDragStart", function()
		if not self.db.profile.locked then
			this:GetParent():StartMoving()
		end
	end)
	f.menuButton:SetScript("OnDragStop", function()
		if not self.db.profile.locked then
			this:GetParent():StopMovingOrSizing()
			self:SavePosition()
		end
	end)
	f.menuButton:RegisterForClicks("RightButtonUp")
	f.menuButton:SetScript("OnClick", function()
		if dewdrop:IsOpen(f) then
			dewdrop:Close()
		else
			dewdrop:Open(f, 'children', function()
				dewdrop:FeedAceOptionsTable(self.core.options)
			end)
		end
	end)
	f.menuButton:SetPoint("TOPLEFT", f, "TOPLEFT", 5, -5)

	f.headerBar = f:CreateTexture(nil, "ARTWORK")
	f.headerBar:SetWidth(buttonWidth)
	f.headerBar:SetHeight(19)
	f.headerBar:SetTexture("Interface\\AddOns\\School\\Textures\\otravi.tga")
	f.headerBar:SetPoint("TOPLEFT", f, "TOPLEFT", 5, -5)
	f.headerBar:SetGradient("HORIZONTAL", 0.8, 0, 0, 0.4, 0, 0)

	f.headerText = f:CreateFontString(nil, "OVERLAY")
	f.headerText:SetWidth(240)
	f.headerText:SetHeight(12)
	f.headerText:SetPoint("TOPLEFT", f, "TOPLEFT", 6, -8)
	f.headerText:SetFontObject(GameFontNormal)
	f.headerText:SetJustifyH("LEFT")
	f.headerText:SetTextColor(1, 1, 1)
	f.headerText:SetText("School")

	f.closeButton = CreateFrame("Button", nil, f, "UIPanelCloseButton")
	f.closeButton:SetWidth(27)
	f.closeButton:SetHeight(27)
	f.closeButton:SetPoint("TOPRIGHT", f) 

	f.scroll = CreateFrame("ScrollFrame", "School_ItemsScrollFrame", f, "FauxScrollFrameTemplate")
	f.scroll:SetScript("OnVerticalScroll", function()
		FauxScrollFrame_OnVerticalScroll(19, function()
			self:UpdateResults()
		end)
	end)
	f.scroll:SetPoint("TOPLEFT", f, "TOPLEFT", 0, -23)
	f.scroll:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -28, 5)

	self.frames.main = f

	self.frames.main.items = {}

	local p
    
    local detailTooltip = {}
    if (self.db.profile.detailPosition == "left") then
        detailTooltip.point = "BOTTOMRIGHT"
        detailTooltip.relativePoint = "BOTTOMLEFT"
    else
        detailTooltip.point = "BOTTOMLEFT"
        detailTooltip.relativePoint = "BOTTOMRIGHT"
    end
    
	for i = 1, self.resultRows do
		local f = CreateFrame("Button", nil, self.frames.main)
		f.schoolID = nil
		f:SetWidth(buttonWidth)
		f:SetHeight(18)
		f:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		f:SetScript("OnEnter", function()
			if this:IsVisible() then
				self:ShowItemTooltip(this.schoolID, this.icon:GetTexture())
				this.highlight:Show()
			end
		end)
		f:SetScript("OnLeave", function()
			self.tooltips.show:Hide()
			this.highlight:Hide()
		end)

		f:SetScript("OnClick", function()
			if this:IsVisible() then
				if arg1 == "LeftButton" then
					if dewdrop:IsOpen(f) then
						dewdrop:Close()
					else
                        if (not self.schoolResults) then return end
                        local item = self.schoolResults[f.schoolID]

						-- dewdrop:Open(f, 'point', "TOPLEFT", 'relativePoint', "TOPRIGHT", 'children', function()
                        -- dewdrop:Open(self.frames.main, 'point', "BOTTOMRIGHT", 'relativePoint', "BOTTOMLEFT", 'children', function()
                        dewdrop:Open(self.frames.main, 'point', detailTooltip.point, 'relativePoint', detailTooltip.relativePoint, 'children', function()
							dewdrop:AddLine(
								'isTitle', true,
								-- 'text', f.name:GetText(),
                                'text', item.fullname,
								'icon', f.icon:GetTexture()
							)
                            dewdrop:AddLine(
                                'text', "Cost" .. " " .. abacus:FormatMoneyExtended(item.cost.money, true),
                                'isClickable', false
                            )
                            
                            if (GetMoney() < item.cost.money) then
                                dewdrop:AddLine(
                                    'text', crayon:Red("You can't afford this."),
                                    'isClickable', false
                                )
                            end
                            
                            dewdrop:AddLine(
                                'isTitle', true,
                                'isClickable', false
                            )
                            
                            if (item.trainers ~= nil) then
                                dewdrop:AddLine(
                                    'isTitle', true,
                                    'text', "Trainers",
                                    'isClickable', false
                                )
                                for tk, tv in pairs(item.trainers) do
                                    if (tv ~= nil) then
                                        local trainer = ""
                                        if (not tv.name) then trainer = "Unknown"
                                        else trainer = tv.name end
                                        if (not tv.subzone) then trainer = trainer .. " at Unknown"
                                        else trainer = trainer .. " " .. "at" .. " " .. tv.subzone end
                                        if (not tv.zone) then trainer = trainer .. " in Unknown"
                                        else trainer = trainer .. " " .. "in" .. " " .. tv.zone end
                                        
                                        dewdrop:AddLine(
                                            'text', trainer,
                                            'isClickable', false
                                        )
                                    end
                                end

                                dewdrop:AddLine(
                                    'isTitle', true,
                                    'isClickable', false
                                )
                            end
                            
							dewdrop:AddLine(
								-- 'text', L["Close window"],
                                'text', "Close window",
								'func', function() dewdrop:Close() end
							)
						end)
					end
				end
			end
		end)

		f.icon = f:CreateTexture(nil, "OVERLAY")
		f.icon:SetWidth(18)
		f.icon:SetHeight(18)
		f.icon:SetPoint("LEFT", f, "LEFT", 0, 0)

		f.highlight = f:CreateTexture(nil, "ARTWORK")
		f.highlight:SetWidth(buttonWidth)
		f.highlight:SetHeight(18)
		f.highlight:SetTexture("Interface\\HelpFrame\\HelpFrameButton-Highlight")
		f.highlight:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
		f.highlight:SetTexCoord(0, 1, 0, 0.578125)
		f.highlight:Hide()

		f.name = f:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		f.name:SetWidth(buttonWidth)
		f.name:SetHeight(18)
		f.name:SetJustifyH("LEFT")
		f.name:SetPoint("LEFT", f, "LEFT", 19, 0)

		if i == 1 then
			f:SetPoint("TOPLEFT", self.frames.main, "TOPLEFT", 6, -25)
		else
			f:SetPoint("TOPLEFT", p, "BOTTOMLEFT", 0, -1)
		end

		table.insert(self.frames.main.items, f)
		p = f
	end
    
    local f = CreateFrame("GameTooltip", "School_TooltipShow", UIParent, "GameTooltipTemplate")

	f.icon = f:CreateTexture(nil, "ARTWORK")
	f.icon:SetWidth(39)
	f.icon:SetHeight(39)

	self.tooltips.show = f

	self:ToggleAdvancedMode(self.db.profile.advancedMode)
	self:SetBackdropAlpha()
	self:RestorePosition()
	-- self:UpdateResults()
end

function SchoolUI:SetBackdropAlpha()
	if not self.frames then return end

	self.frames.main:SetBackdropColor(0, 0, 0, self.db.profile.backgroundAlpha)
end

function SchoolUI:SavePosition()
	if not self.frames then return end

	local x, y, s = self.frames.main:GetLeft(), self.frames.main:GetTop(), self.frames.main:GetEffectiveScale()

	x, y = x * s, y * s

	self.db.profile.mainPositionX = x
	self.db.profile.mainPositionY = y
end

function SchoolUI:RestorePosition()
	if not self.frames then return end

	local x, y, s = self.db.profile.mainPositionX, self.db.profile.mainPositionY, self.frames.main:GetEffectiveScale()

	if not x or not y then
		return
	end

	x, y = x / s, y / s

	self.frames.main:ClearAllPoints()
	self.frames.main:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end

function SchoolUI:ShowItemTooltip(schoolID, icon)
    if (not self.schoolResults) then return end
    if (not self.schoolResults[schoolID]) then return end
    
    local result = self.schoolResults[schoolID]
    local resultInfo = result.fullname .. "\n" .. "Category: " .. result.category .. "\n" .. "Cost: " .. abacus:FormatMoneyShort(result.cost.money, true)
    if (GetMoney() < result.cost.money) then
        resultInfo = resultInfo .. "\n" .. crayon:Red("You can't afford this.")
    end
    
    self:PlaceTooltip()
    self.tooltips.show:SetOwner(UIParent, "ANCHOR_PRESERVE")
    self.tooltips.show:SetText(resultInfo)
    self.tooltips.show:Show()
    self.tooltips.show.icon:SetTexture(icon)
end

function SchoolUI:FetchResults()
    self:Debug("FetchResults")
    local school = School:GotoSchool(true)
    if (not school) then return end

    local show = {}
    local money = GetMoney()
    local spells = school["spells"]
    local trainers = school["trainers"]

    if (spells ~= nil) then
        for k,v in pairs(spells) do
            if (v ~= nil) then
                -- self:LevelDebug(3, "Key ==> %s", k)
                for vk, vv in pairs(v) do
                    -- self:LevelDebug(3, "Value Key ==> %s", vk) 
                    if (vv ~= nil) then
                        local spell = vv
                        --[[
                        ["Shadow Word: Pain"] = {
                            ["Rank 1"] = {
                                ["Matron Arena"] = {
                                    ["subzone"] = "The Sunspire",
                                    ["name"] = "Matron Arena",
                                    ["zone"] = "Eversong Woods",
                                },
                            },
                        },
                        --]]
                        
                        if (trainers ~= nil) then
                            self:LevelDebug(3, "Found trainers.")
                            if (trainers[spell.name] ~= nil) then
                                self:LevelDebug(3, "Found trainers for " .. spell.name)
                                if (trainers[spell.name][spell.subtext] ~= nil) then
                                    self:LevelDebug(3, "Found trainers for " .. spell.name .. " Rank: " .. spell.subtext)
                                    if (not spell.trainers) then spell.trainers = {} end
                                    for tk, tv in pairs(trainers[spell.name][spell.subtext]) do
                                        if (tv ~= nil) then
                                            table.insert(spell.trainers, tv)
                                        end
                                    end
                                end
                            end
                        end
                        
                        --[[
                        if (trainers ~= nil) and (trainers[spell.name] ~= nil) and (trainers[spell.name][spell.subtext] ~= nil) then
                            if (not spell.trainers) then spell.trainers = {} end
                            for tk, tv in pairs(trainers[spell.name][spell.subtext]) do
                                if (tv ~= nil) then table.insert(spell.trainers, tv) end
                            end
                        end
                        --]]

                        if (not self.db.profile.ignoreCost) then
                            if (spell.cost.money ~= nil) and (money >= spell.cost.money) then table.insert(show, spell) end
                        else
                            table.insert(show, spell)
                        end
                    end
                end
            end
        end
    end
    
    --[[
    for k,v in pairs(show) do
        if (v ~= nil) then
            for vk, vv in pairs(v) do
                if (vv ~= nil) then
                    -- self:LevelDebug(3, "Value Key ==> ", vk)
                end
            end
        end
    end
    --]]
    
    return show
end

function SchoolUI:UpdateResults()
    self:Debug("UpdateResults")
	if not self.frames then return end

    self.schoolResults = self:FetchResults()
    if (not self.schoolResults) then return end
    
    self.studentMoney = GetMoney()
    self.numResults = table.maxn(self.schoolResults)

	-- FauxScrollFrame_Update(self.frames.main.scroll, table.maxn(self.schoolResults), self.resultRows, 19);
    FauxScrollFrame_Update(self.frames.main.scroll, self.numResults, self.resultRows, 19);

	for i = 1, self.resultRows do
		local index = i + FauxScrollFrame_GetOffset(self.frames.main.scroll)

		if (index <= table.maxn(self.schoolResults)) then
            local item = self.schoolResults[index]
            self.frames.main.items[i].schoolID = index
            self.frames.main.items[i].icon:SetTexture(item.icon)
            local line = item.fullname .. "\t" .. abacus:FormatMoneyFull(item.cost.money, true)
            self.frames.main.items[i].name:SetText(line)

            if (GetMoney() < item.cost.money) then
                self.frames.main.items[i].icon:SetVertexColor(1,0,0)
            else 
                self.frames.main.items[i].icon:SetVertexColor(1,1,1)
            end

			self.frames.main.items[i]:Show()
		else
			self.frames.main.items[i]:Hide()
		end
	end

	-- self.frames.main.headerText:SetText(string.format("School (%d)", table.maxn(self.schoolResults)))
    self.frames.main.headerText:SetText(string.format("School (%d)", self.numResults))
end
    