local DATABASE_VERSION = 1
local _, _, REVISION = string.find("$Revision: 17474 $", "(%d+)")

-- lua 5.1 check
if(not loadstring("return function(...) return ... end")  and true or false) then
    string.gmatch = string.gfind;
    math.fmod = math.mod;    
    table.maxn = table.getn;
    IsPassiveSpell = IsSpellPassive
end

if AceLibrary:HasInstance("AceDebug-2.0") then
    School = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0", "AceDebug-2.0");
    School:SetDebugging(false)
    School:SetDebugLevel(3)
else
	School = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0");
	function School:Debug() end
    function School:LevelDebug() end
    function School:IsDebugging() return false end
    function School:SetDebugging() end
    function School:GetDebugLevel() end
    function School:SetDebugLevel() end
end

School.revision = tonumber(REVISION)
local lang = AceLibrary("AceLocale-2.2"):new("School")
-- local toolbox = AceLibrary("Toolbox-1.0")
-- local dewdrop = AceLibrary("Dewdrop-2.0")
-- local deformat = AceLibrary("Deformat-2.0")
-- local tooltip = AceLibrary("Gratuity-2.0")

School.debugFrame = ChatFrame1

School.defaults = {
                   showInChat = true,
                   debugEnabled = false,
                   debugLevel = nil,
}

School.options = {
                  type = 'group',
                  name = lang["School"],
                  desc = lang["What do you want to learn today?"],
                  args = {
                  },
                  handler = School
}

School.options.args.standby = nil

School:RegisterChatCommand({lang["SLASHCMD_LONG"], lang["SLASHCMD_SHORT"]}, School.options)

School:RegisterDB("SchoolDB", "SchoolCharDB")
School:RegisterDefaults("profile", School.defaults)

if AceLibrary:HasInstance("AceDB-2.0") then
    School.charid = AceLibrary("AceDB-2.0").CHAR_ID
    School.realmid = AceLibrary("AceDB-2.0").REALM_ID
    School.classid = AceLibrary("AceDB-2.0").CLASS_ID
    School.faction = AceLibrary("AceDB-2.0").FACTION
    School.realm = AceLibrary("AceDB-2.0").REALM
    School.name = AceLibrary("AceDB-2.0").NAME
else
    School.charid = ""
    School.realmid = ""
    School.classid = ""
    School.faction = ""
    School.realm = ""
    School.name = UnitName("player")
end

--[[
-------------------------------------------------------
Module prototype 
-------------------------------------------------------
--]]                         

if AceLibrary:HasInstance("AceDebug-2.0") then
    School:SetModuleMixins("Toolbox-1.0", "AceEvent-2.0", "AceDebug-2.0")
else 
    School:SetModuleMixins("Toolbox-1.0", "AceEvent-2.0")
    
    function School.modulePrototype:Debug() end
    function School.modulePrototype:LevelDebug() end
    function School.modulePrototype:IsDebugging() return false end
    function School.modulePrototype:SetDebugging() end
    function School.modulePrototype:GetDebugLevel() end
    function School.modulePrototype:SetDebugLevel() end
end

School.modulePrototype.core = School
School.modulePrototype.canToggle = false
School.modulePrototype.revision = 0
School.modulePrototype.debugFrame = School.debugFrame
    
function School:OnInitialize()
	self.contextMenuItems = {}

	for name, module in self:IterateModules() do
		self:RegisterModule(name, module)
	end
end

function School:OnEnable()
    -- self:RegisterEvent("School_Teacher_Changed")
    -- self:RegisterEvent("School_Student_Changed")
    self:RegisterEvent("School_Data_Changed")
    
    self:RegisterEvent("CHARACTER_POINTS_CHANGED", "GotoSchool")
    self:RegisterEvent("PLAYER_LEVEL_UP", "GotoSchool")
end

function School:OnDisable()
end

function School:RegisterModule(name, module)
	if module.revision and module.revision > self.revision then
		self.revision = module.revision
	end

	if module.internalModule then return end

	self:RegisterDefaults(name, "profile", module.defaults or {})
	module.db = self:AcquireDBNamespace(name)

	if module.commands then
		for k, v in pairs(module.commands) do
			self.options.args[k] = v
		end
	end

	if module.canToggle or module.options then
		local m = module
		local options = {
			type = "group",
			name = name,
			desc = string.format(lang["Options for %s module."], name),
			order = 200,
			args = {},
		}

		if module.canToggle then
			options.args.enabled = {
				type = "toggle",
				name = lang["Enabled"],
				desc = lang["Enable or disable this module."],
				get = function() return m.core:IsModuleActive(m) end,
				set = function() m.core:ToggleModuleActive(m) end,
				order = 1,
			}
		end

		if module.options then
			if module.canToggle then
				options.args.spacerHeader = {
					type = "header",
					order = 2,
				}
			end

			for k, v in pairs(module.options) do
				options.args[k] = v
			end
		end

		self.options.args[string.lower(name)] = options
	end
end    

function School:GetData()
    return self.school
end

function School:School_Teacher_Changed()
    self:LevelDebug(3, "School_Teacher_Changed")
    self:BuildTeacher()
end

function School:School_Student_Changed()
    self:LevelDebug(3, "School_Student_Changed")
    self:BuildStudent()
end

function School:School_Data_Changed()
    self:LevelDebug(3, "School_Data_Changed")
    self:BuildSchool()
end

function School:BuildTeacher()
    -- self:LevelDebug(3, "BuildTeacher")
    local teacher = self:GetModule("Teacher")
    if (not teacher) then return end
    self.teacher = teacher:GetData(false)
end

function School:BuildStudent()
    -- self:LevelDebug(3, "BuildStudent")
    local student = self:GetModule("Student")
    if (not student) then return end
    self.student = student:Rebuild()
end

function School:BuildSchool()
    -- self:Debug("BuildSchool")
    
    if (not self.teacher) then self:BuildTeacher() end
    if (not self.student) then self:BuildStudent() end
    
    local school = {}
    local teacher = self.teacher
    local student = self.student
    
    if (not teacher) then return end
    if (not student) then return end

    student.level = UnitLevel("player")
    -- student.level = 20
    
    if (not teacher.spells) then return end

    local apprentice = {}
    apprentice.primary = {
                          ["Apprentice Alchemist"] = true,
                          ["Apprentice Blacksmith"] = true,
                          ["Apprentice Enchanting"] = true,
                          ["Apprentice Engineer"] = true,
                          ["Apprentice Herbalist"] = true,
                          ["Apprentice Jewelcrafting"] = true,
                          ["Apprentice Miner"] = true,
                          ["Apprentice Leatherworking"] = true,
                          ["Apprentice Skinning"] = true,
                          ["Apprentice Tailor"] = true,
    }
    apprentice.secondary = {
                            ["Apprentice Cook"] = true,
                            ["Apprentice First Aid"] = true,
                            ["Apprentice Fishing"] = true,
    }
    
    for k,v in pairs(teacher.spells) do
        if (v ~= nil) then
            -- self:LevelDebug(3, "Key ==> %s", k)
            for vk, vv in pairs(v) do
                if (vv ~= nil) then
                    -- self:LevelDebug(3, "Value Key ==> %s", vk)
                    for vvk, vvv in pairs(vv) do
                        if (vvv ~= nil) then
                            -- self:LevelDebug(3, "Value Value Key ==> %s", vvk)
                            local spell = vvv
                            -- self:Debug("spell.name ==> %s", spell.name)
                            
                            --[[
                            ["type"] = "unavailable",
							["description"] = "How to find and gather more difficult to harvest herbs.   Gives a potential herbalism skill of 150.",
							["cost"] = {
								["money"] = 500,
								["cost2"] = 0,
								["cost1"] = 0,
							},
							["subtext"] = "None",
							["learnspell"] = 1,
							["name"] = "Journeyman Herbalist",
							["category"] = "Development Skills",
							["icon"] = "Interface\\Icons\\Trade_Herbalism",
							["level"] = 0,
							["expanded"] = 1,
							["req"] = {
								["skill"] = "Herbalism",
								["rank"] = 50,
							},
                            --]]

                            local continue = true
                            
                            if (spell.type == "used") then continue = false end
                            
                            if (spell.type == "unavailable") and continue then
                                if (student.level < spell.level) then continue = false end
                                
                                if (spell.ability ~= nil) and continue then
                                    -- Check ability requirements
                                end
                                
                                if (spell.req ~= nil) and continue then
                                    -- self:LevelDebug(3, "Check spell.req")
                                    continue = false
                                    if (spell.req.skill ~= nil) then
                                        if (student.skills[spell.req.skill] ~= nil) then
                                            -- student.skills[spell.req.skill].rank = 50
                                            if (not spell.req.rank) or (spell.req.rank == 0) then 
                                                continue = true
                                            else
                                                -- local msg = string.format("You need to have %s (%s)", spell.req.skill, spell.req.rank)
                                                -- msg = msg .. "\n" .. string.format("You have %s (%s)", student.skills[spell.req.skill].name, student.skills[spell.req.skill].rank)
                                                -- self:LevelDebug(3, msg)
                                                if (student.skills[spell.req.skill].rank >= spell.req.rank) then
                                                    -- self:LevelDebug(3, "You qualify for %s", spell.req.skill)
                                                    continue = true
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                            
                            if (continue) then
                                if (student.professions ~= nil) then
                                    -- self:Debug("Found student professions.")
                                    
                                    local primary = student.professions.primary
                                    local secondary = student.professions.secondary
                                    -- self:Debug("primary.count ==> %s", primary.count)
                                    -- self:Debug("secondary.count ==> %s", secondary.count)                                    

                                    local primaryContinue = true
                                    local secondaryContinue = true
                                    
                                    -- self:Debug("spell.name ==> %s", spell.name)
                                    if (primary.count == 2) then
                                        -- self:Debug("You already have 2 primary skills")
                                        if (apprentice.primary[spell.name]) then primaryContinue = false end
                                    end
                                    
                                    if (secondary.count == 3) then
                                        -- self:Debug("You already have 3 secondary skills.")
                                        if (apprentice.secondary[spell.name]) then secondaryContinue = false end
                                    end
                                    
                                    continue = primaryContinue and secondaryContinue
                                end
                            end
                            
                            if (continue) then
                                if (teacher.trainers ~= nil) then
                                    spell.trainers = {}
                                    local trainers = nil
                                    if (teacher.trainers[spell.name] ~= nil) and (teacher.trainers[spell.name][spell.subtext] ~= nil) then
                                        trainers = teacher.trainers[spell.name][spell.subtext]
                                        for tk, tv in pairs(trainers) do
                                            if (tv ~= nil) then 
                                                table.insert(spell.trainers, tv) 
                                            end
                                        end
                                    end
                                end

                                if (not school.spells) then school.spells = {} end
                                if (not school.spells[spell.category]) then school.spells[spell.category] = {} end
                                if (not school.spells[spell.category][spell.name]) then school.spells[spell.category][spell.name] = {} end
                                school.spells[spell.category][spell.name] = spell
                            end
                        end
                    end
                end
            end
        end
    end

    --[[
    if (school ~= nil) then
        for k,v in pairs(school) do
            self:LevelDebug(3, "Key ==> %s", k)
        end
    end
    --]]
    
    self.school = school
    return self.school
end

function School:GotoSchool(force)
    -- self:Debug("GotoSchool")
    
    if (type(force) == "boolean") then
        if (force == true) then
            self:BuildTeacher()
            self:BuildStudent()
        end
    end
    
    self.db.char.school = self:BuildSchool()
    return self.db.char.school
end

function School:Test()
    local toolbox = AceLibrary("Toolbox-1.0")
    local ts = "This is a test string."
    local rs = toolbox:Reverse(ts)
    
    self:LevelDebug(3, "Test: %s\nResult: %s", ts, rs)
    
    -- local tn = 14334453.233
    local tn = 1234567890.1234567890
    local cs = toolbox:Commify(tn)
    
    self:LevelDebug(3, "Test: %d\nResult: %s", tn, cs)    
end
